@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1996-2023. All Rights Reserved.
+%% Copyright Ericsson AB 1996-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -19,6 +19,8 @@
 %%
 -module(group).
 
+-include_lib("kernel/include/logger.hrl").
+
 %% A group leader process for user io.
 %% This process receives input data from user_drv in this format
 %%   {Drv,{data,unicode:charlist()}}
@@ -44,14 +46,26 @@ server(Ancestors, Drv, Shell, Options) ->
     process_flag(trap_exit, true),
     _ = [put('$ancestors', Ancestors) || Shell =/= {}],
     edlin:init(),
-    put(line_buffer, proplists:get_value(line_buffer, Options, group_history:load())),
     put(read_mode, list),
     put(user_drv, Drv),
     ExpandFun = normalize_expand_fun(Options, fun edlin_expand:expand/2),
     put(expand_fun, ExpandFun),
-    put(echo, proplists:get_value(echo, Options, true)),
+    Echo = proplists:get_value(echo, Options, true),
+    put(echo, Echo),
+    Dumb = proplists:get_value(dumb, Options, false),
+    put(dumb, Dumb),
     put(expand_below, proplists:get_value(expand_below, Options, true)),
 
+    DefaultGroupHistory =
+        case not get(echo) of
+            true ->
+                [];
+            false ->
+                group_history:load()
+        end,
+
+    put(line_buffer, proplists:get_value(line_buffer, Options, DefaultGroupHistory)),
+
     server_loop(Drv, start_shell(Shell), []).
 
 whereis_shell() ->
@@ -467,13 +481,21 @@ getopts(Drv,Buf) ->
 %%	{error,What,NewSaveBuffer}
 
 get_password_chars(Drv,Shell,Buf) ->
-    case get_password_line(Buf, Drv, Shell) of
-	{done, Line, Buf1} ->
-	    {ok, Line, Buf1};
-	interrupted ->
-	    {error, {error, interrupted}, []};
-	terminated ->
-	    {exit, terminated}
+    case get(echo) of
+        true ->
+            case get_password_line(Buf, Drv, Shell) of
+                {done, Line, Buf1} ->
+                    {ok, Line, Buf1};
+                interrupted ->
+                    {error, {error, interrupted}, []};
+                terminated ->
+                    {exit, terminated}
+            end;
+        false ->
+            %% Echo needs to be set to true, otherwise the
+            %% password will be printed to the shell and we
+            %% do not want that.
+            {error, {error, enotsup}, []}
     end.
 
 get_chars_n(Prompt, M, F, Xa, Drv, Shell, Buf, Encoding) ->
@@ -490,19 +512,18 @@ get_chars_line(Prompt, M, F, Xa, Drv, Shell, Buf, Encoding) ->
     get_chars_loop(Pbs, M, F, Xa, Drv, Shell, Buf, start, [], Encoding).
 
 get_chars_loop(Pbs, M, F, Xa, Drv, Shell, Buf0, State, LineCont0, Encoding) ->
-    Result = case get(echo) of
+    Result = case not(get(dumb)) andalso get(echo) of
                  true ->
                      get_line(Buf0, Pbs, LineCont0, Drv, Shell, Encoding);
                  false ->
-                     %% get_line_echo_off only deals with lists,
-                     %% so convert to list before calling it.
-                     get_line_echo_off(cast(Buf0, list, Encoding), Pbs, Drv, Shell)
+                     get_line_echo_off(Buf0, Encoding, Pbs, Drv, Shell)
              end,
     case Result of
         {done,LineCont1,Buf} ->
             get_chars_apply(Pbs, M, F, Xa, Drv, Shell, append(Buf, [], Encoding),
                             State, LineCont1, Encoding);
-
+        {no_translation, unicode, latin1} ->
+            {error,{error,{no_translation, unicode, latin1}}, []};
         interrupted ->
             {error,{error,interrupted},[]};
         terminated ->
@@ -512,11 +533,13 @@ get_chars_loop(Pbs, M, F, Xa, Drv, Shell, Buf0, State, LineCont0, Encoding) ->
 get_chars_apply(Pbs, M, F, Xa, Drv, Shell, Buf, State0, LineCont, Encoding) ->
     %% multi line support means that we should not keep the state
     %% but we need to keep it for oldshell mode
-    {State, Line} = case get(echo) of
+    {State, Line} = case not(get(dumb)) andalso get(echo) of
                         true -> {start, edlin:current_line(LineCont)};
                         false -> {State0, LineCont}
                     end,
     case catch M:F(State, cast(Line,get(read_mode), Encoding), Encoding, Xa) of
+        {stop,eof,_} ->
+            {ok,eof,eof};
         {stop,Result,eof} ->
             {ok,Result,eof};
         {stop,Result,Rest} ->
@@ -543,20 +566,27 @@ get_chars_apply(Pbs, M, F, Xa, Drv, Shell, Buf, State0, LineCont, Encoding) ->
     end.
 
 get_chars_n_loop(Pbs, M, F, Xa, Drv, Shell, Buf0, State, Encoding) ->
-    try M:F(State, cast(Buf0, get(read_mode), Encoding), Encoding, Xa) of
-        {stop,Result,Rest} ->
-            {ok, Result, append(Rest,[],Encoding)};
-        State1 ->
-            case get_chars_echo_off(Pbs, Drv, Shell) of
-                interrupted ->
-                    {error,{error,interrupted},[]};
-                terminated ->
-                    {exit,terminated};
-                Buf ->
-                    get_chars_n_loop(Pbs, M, F, Xa, Drv, Shell, Buf, State1, Encoding)
+    case check_encoding(Buf0, Encoding) of
+        false ->
+            {error,{error,{no_translation,unicode,Encoding}},[]};
+        true ->
+            try M:F(State, cast(Buf0, get(read_mode), Encoding), Encoding, Xa) of
+                {stop,eof,_} ->
+                    {ok, eof, eof};
+                {stop,Result,Rest} ->
+                    {ok, Result, append(Rest,[],Encoding)};
+                State1 ->
+                    case get_chars_echo_off(Pbs, Drv, Shell) of
+                        interrupted ->
+                            {error,{error,interrupted},[]};
+                        terminated ->
+                            {exit,terminated};
+                        Buf ->
+                            get_chars_n_loop(Pbs, M, F, Xa, Drv, Shell, Buf, State1, Encoding)
+                    end
+            catch _:_ ->
+                    {error,{error,err_func(M, F, Xa)},[]}
             end
-    catch _:_ ->
-            {error,{error,err_func(M, F, Xa)},[]}
     end.
 
 %% Convert error code to make it look as before
@@ -797,9 +827,14 @@ more_data(What, Cont0, Drv, Shell, Ls, Encoding) ->
             get_line1(edlin:edit_line(eof, Cont0), Drv, Shell, Ls, Encoding);
         {io_request,From,ReplyAs,Req} when is_pid(From) ->
             {more_chars,Cont,_More} = edlin:edit_line([], Cont0),
-            send_drv_reqs(Drv, edlin:erase_line()),
-            io_request(Req, From, ReplyAs, Drv, Shell, []), %WRONG!!!
-            send_drv_reqs(Drv, edlin:redraw_line(Cont)),
+            case application:get_env(stdlib, shell_redraw_prompt_on_output, true) of
+                true ->
+                    send_drv_reqs(Drv, edlin:erase_line()),
+                    io_request(Req, From, ReplyAs, Drv, Shell, []),
+                    send_drv_reqs(Drv, edlin:redraw_line(Cont));
+                false ->
+                    io_request(Req, From, ReplyAs, Drv, Shell, [])
+            end,
             get_line1({more_chars,Cont,[]}, Drv, Shell, Ls, Encoding);
         {reply,{From,ReplyAs},Reply} ->
             %% We take care of replies from puts here as well
@@ -816,11 +851,25 @@ more_data(What, Cont0, Drv, Shell, Ls, Encoding) ->
             get_line1(edlin:edit_line([], Cont0), Drv, Shell, Ls, Encoding)
     end.
 
-get_line_echo_off(Chars, Pbs, Drv, Shell) ->
+get_line_echo_off(Chars, ToEnc, Pbs, Drv, Shell) ->
     send_drv_reqs(Drv, [{put_chars, unicode,Pbs}]),
-    get_line_echo_off1(edit_line(Chars,[]), Drv, Shell).
+    case get_line_echo_off1(edit_line(Chars,[]), Drv, Shell) of
+        {done, Line, _Rest} = Res when ToEnc =:= latin1 ->
+            case check_encoding(Line, ToEnc) of
+                false ->
+                    {no_translation, unicode, ToEnc};
+                true ->
+                    Res
+            end;
+        Res ->
+            Res
+    end.
 
-get_line_echo_off1({Chars,[]}, Drv, Shell) ->
+get_line_echo_off1({Chars,[],Rs}, Drv, Shell) ->
+    case get(echo) of
+        true -> send_drv_reqs(Drv, Rs);
+        false -> skip
+    end,
     receive
 	{Drv,{data,Cs}} ->
 	    get_line_echo_off1(edit_line(cast(Cs, list), Chars), Drv, Shell);
@@ -828,11 +877,11 @@ get_line_echo_off1({Chars,[]}, Drv, Shell) ->
 	    get_line_echo_off1(edit_line(eof, Chars), Drv, Shell);
 	{io_request,From,ReplyAs,Req} when is_pid(From) ->
 	    io_request(Req, From, ReplyAs, Drv, Shell, []),
-	    get_line_echo_off1({Chars,[]}, Drv, Shell);
+	    get_line_echo_off1({Chars,[],[]}, Drv, Shell);
         {reply,{From,ReplyAs},Reply} when From =/= undefined ->
             %% We take care of replies from puts here as well
             io_reply(From, ReplyAs, Reply),
-            get_line_echo_off1({Chars,[]},Drv, Shell);
+            get_line_echo_off1({Chars,[],[]},Drv, Shell);
 	{'EXIT',Drv,interrupt} ->
 	    interrupted;
 	{'EXIT',Drv,_} ->
@@ -842,9 +891,12 @@ get_line_echo_off1({Chars,[]}, Drv, Shell) ->
     end;
 get_line_echo_off1(eof, _Drv, _Shell) ->
     {done,eof,eof};
-get_line_echo_off1({Chars,Rest}, _Drv, _Shell) ->
+get_line_echo_off1({Chars,Rest,Rs}, Drv, _Shell) ->
+    case get(echo) of
+        true -> send_drv_reqs(Drv, Rs);
+        false -> skip
+    end,
     {done,lists:reverse(Chars),case Rest of done -> []; _ -> Rest end}.
-
 get_chars_echo_off(Pbs, Drv, Shell) ->
     send_drv_reqs(Drv, [{insert_chars, unicode,Pbs}]),
     get_chars_echo_off1(Drv, Shell).
@@ -879,22 +931,26 @@ get_chars_echo_off1(Drv, Shell) ->
 %% - ^d in posix/icanon mode: eof, delete-forward in edlin
 %% - ^r in posix/icanon mode: reprint (silly in echo-off mode :-))
 %% - ^w in posix/icanon mode: word-erase (produces a beep in edlin)
-edit_line(eof, []) ->
+edit_line(Input, State) ->
+    edit_line(Input, State, []).
+edit_line(eof, [], _) ->
     eof;
-edit_line(eof, Chars) ->
-    {Chars,eof};
-edit_line([],Chars) ->
-    {Chars,[]};
-edit_line([$\r,$\n|Cs],Chars) ->
-    {[$\n | Chars], remainder_after_nl(Cs)};
-edit_line([NL|Cs],Chars) when NL =:= $\r; NL =:= $\n ->
-    {[$\n | Chars], remainder_after_nl(Cs)};
-edit_line([Erase|Cs],[]) when Erase =:= $\177; Erase =:= $\^H ->
-    edit_line(Cs,[]);
-edit_line([Erase|Cs],[_|Chars]) when Erase =:= $\177; Erase =:= $\^H ->
-    edit_line(Cs,Chars);
-edit_line([Char|Cs],Chars) ->
-    edit_line(Cs,[Char|Chars]).
+edit_line(eof, Chars, Rs) ->
+    {Chars,eof, lists:reverse(Rs)};
+edit_line([],Chars, Rs) ->
+    {Chars,[],lists:reverse(Rs)};
+edit_line([$\r,$\n|Cs],Chars, Rs) ->
+    {[$\n | Chars], remainder_after_nl(Cs), lists:reverse([{put_chars, unicode, "\n"}|Rs])};
+edit_line([NL|Cs],Chars, Rs) when NL =:= $\r; NL =:= $\n ->
+    {[$\n | Chars], remainder_after_nl(Cs), lists:reverse([{put_chars, unicode, "\n"}|Rs])};
+edit_line([Erase|Cs],[], Rs) when Erase =:= $\177; Erase =:= $\^H ->
+    edit_line(Cs,[], Rs);
+edit_line([Erase|Cs],[_|Chars], Rs) when Erase =:= $\177; Erase =:= $\^H ->
+    edit_line(Cs,Chars, [{delete_chars, -1}|Rs]);
+edit_line([CtrlChar|Cs],Chars, Rs) when CtrlChar < 32 ->
+    edit_line(Cs,Chars,Rs);
+edit_line([Char|Cs],Chars, Rs) ->
+    edit_line(Cs,[Char|Chars], [{put_chars, unicode, [Char]}|Rs]).
 
 remainder_after_nl("") -> done;
 remainder_after_nl(Cs) -> Cs.
@@ -965,7 +1021,12 @@ save_line_buffer("\n", Lines) ->
 save_line_buffer(Line, [Line|_Lines]=Lines) ->
     save_line_buffer(Lines);
 save_line_buffer(Line, Lines) ->
-    group_history:add(Line),
+    try
+        group_history:add(Line)
+    catch E:R:ST ->
+            ?LOG_ERROR(#{ msg => "Failed to write to shell history",
+                          error => {E, R, ST} })
+    end,
     save_line_buffer([Line|Lines]).
 
 save_line_buffer(Lines) ->
@@ -1000,7 +1061,7 @@ get_password_line(Chars, Drv, Shell) ->
 get_password1({Chars,[]}, Drv, Shell) ->
     receive
 	{Drv,{data,Cs}} ->
-	    get_password1(edit_password(Cs,cast(Chars,list)),Drv,Shell);
+	    get_password1(edit_password(cast(Cs,list),Chars),Drv,Shell);
 	{io_request,From,ReplyAs,Req} when is_pid(From) ->
 	    io_request(Req, From, ReplyAs, Drv, Shell, []), %WRONG!!!
 	    %% I guess the reason the above line is wrong is that Buf is
@@ -1046,6 +1107,8 @@ cast(eof, _, _) ->
     eof;
 cast(L, binary, ToEnc) ->
     unicode:characters_to_binary(L, utf8, ToEnc);
+cast(L, list, _ToEnc) when is_list(L) ->
+    L;
 cast(L, list, _ToEnc) ->
     unicode:characters_to_list(L, utf8).
 
@@ -1053,5 +1116,26 @@ append(eof, [], _) ->
     eof;
 append(eof, L, _) ->
     L;
+append(L, [], _) when is_list(L) ->
+    %% When doing ++ all of L needs to be traversed to check if it is
+    %% a proper list. Since we know it is a proper list we just return
+    %% the list without checking.
+    L;
+append(L, A, _) when is_list(L) ->
+    L ++ A; %% We know L is valid unicode, so we just append the two
 append(B, L, FromEnc) ->
-    unicode:characters_to_list(B, FromEnc) ++ L.
+    append(unicode:characters_to_list(B, FromEnc), L, FromEnc).
+
+check_encoding(eof, _) ->
+    true;
+check_encoding(ListOrBinary, unicode) when is_list(ListOrBinary); is_binary(ListOrBinary) ->
+    true;
+check_encoding(List, latin1) when is_list(List) ->
+    is_latin1(List).
+
+is_latin1([H|T]) when 0 =< H, H =< 255 ->
+    is_latin1(T);
+is_latin1([]) ->
+    true;
+is_latin1(_) ->
+    false.
