@@ -162,7 +162,7 @@
 -type oid()                  :: tuple().
 -type cert_id()              :: {SerialNr::integer(), issuer_name()} .
 -type issuer_name()          :: {rdnSequence,[[#'AttributeTypeAndValue'{}]]} .
--type bad_cert_reason()      :: cert_expired | invalid_issuer | invalid_signature | name_not_permitted | missing_basic_constraint | invalid_key_usage | duplicate_cert_in_path |
+-type bad_cert_reason()      :: cert_expired | invalid_issuer | invalid_signature | name_not_permitted | missing_basic_constraint | invalid_key_usage |{key_usage_mismatch, term()}  | duplicate_cert_in_path |
                                 {'policy_requirement_not_met', term()} | {'invalid_policy_mapping', term()} | {revoked, crl_reason()} | invalid_validity_dates | atom().
 
 -type combined_cert()        :: #cert{}.
@@ -229,7 +229,7 @@ pem_entry_decode({'SubjectPublicKeyInfo', Der, _}) ->
             {params, DssParams} = der_decode('DSAParams', Params),
             {der_decode(KeyType, Key0), DssParams};
         'ECPoint' ->
-	    ECCParams = der_decode('EcpkParameters', Params),
+            ECCParams = ec_decode_params(AlgId, Params),
             {#'ECPoint'{point = Key0}, ECCParams}
     end;
 pem_entry_decode({Asn1Type, Der, not_encrypted}) when is_atom(Asn1Type),
@@ -1482,6 +1482,12 @@ cacerts_clear() ->
 %%--------------------------------------------------------------------
 %%% Internal functions
 %%--------------------------------------------------------------------
+ec_decode_params(AlgId, _) when AlgId == ?'id-Ed25519';
+                                AlgId == ?'id-Ed448' ->
+    {namedCurve, AlgId};
+ec_decode_params(_, Params) ->
+    der_decode('EcpkParameters', Params).
+
 default_options([]) ->
     [{rsa_padding, rsa_pkcs1_padding}];
 default_options(Opts) ->
