@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1996-2023. All Rights Reserved.
+%% Copyright Ericsson AB 1996-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -74,7 +74,7 @@ start(Pbs) ->
 
 %% Only two modes used: 'none' and 'search'. Other modes can be
 %% handled inline through specific character handling.
-start(Pbs, {_,{_,_},[]}=Cont) ->
+start(Pbs, {_,{_,[]},[]}=Cont) ->
     %% Skip redraw if the cursor is at the end.
     {more_chars,{line,Pbs,Cont,{normal,none}},[{insert_chars,unicode,multi_line_prompt(Pbs)}]};
 
@@ -88,11 +88,24 @@ start(Pbs, EditState) ->
 keymap() ->
     get(key_map).
 
+-type continuation() :: {line, _, _, _} | {_, _, _}.
+-spec edit_line(unicode:chardata() | eof, continuation()) ->
+          {done, continuation(), Rest :: unicode:chardata(), [user_drv:request()]} |
+          {open_editor | format_expression | history_up | history_down | search,
+           Cs :: unicode:chardata(), continuation(), [user_drv:request()]} |
+          {help | expand | expand_full,
+           Before :: unicode:chardata(),
+           Cs :: unicode:chardata(), continuation(),
+           [user_drv:request()]} |
+          {search_found | search_quit | search_cancel,
+           Cs :: unicode:chardata(), [user_drv:request()]} |
+          {blink | more_chars, continuation(), [user_drv:request()]}.
 edit_line(Cs, {line,P,L,{blink,N_Rs}}) ->
     edit(Cs, P, L, {normal, none}, N_Rs);
 edit_line(Cs, {line,P,L,M}) ->
     edit(Cs, P, L, M, []).
 
+%% Called when in search mode
 edit_line1(Cs, {line,P,L,{blink,N_Rs}}) ->
     edit(Cs, P, L, {normal, none}, N_Rs);
 edit_line1(Cs, {line,P,{B,{[],[]},A},{normal,none}}) ->
