@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1996-2023. All Rights Reserved.
+%% Copyright Ericsson AB 1996-2025. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -263,8 +263,15 @@ server_loop(N0, Eval_0, Bs00, RT, FT, Ds00, History0, Results0) ->
                             [N]),
             server_loop(N0, Eval0, Bs0, RT, FT, Ds0, History0, Results0);
         eof ->
-            fwrite_severity(fatal, <<"Terminating erlang (~w)">>, [node()]),
-            halt()
+            RemoteShell = node() =/= node(group_leader()),
+            case RemoteShell of
+                true ->
+                    exit(Eval0, kill),
+                    terminated;
+                false ->
+                    fwrite_severity(fatal, <<"Terminating erlang (~w)">>, [node()]),
+                    halt()
+            end
     end.
 
 get_command(Prompt, Eval, Bs, RT, FT, Ds) ->
@@ -278,8 +285,24 @@ get_command(Prompt, Eval, Bs, RT, FT, Ds) ->
                       io:scan_erl_exprs(group_leader(), Prompt, {1,1},
                                         [text,{reserved_word_fun,ResWordFun}])
                   of
-                      {ok,Toks,_EndPos} ->
-                          %% NOTE: we can handle function definitions, records and soon type declarations
+                      {ok,Toks0,_EndPos} ->
+                        %% local 'fun' fixer
+                        %% when we parse a 'fun' expression within a shell call or function definition
+                        %% we need to add a local prefix (if the 'fun' expression did not have a module specified)
+                        LocalFunFixer = fun F([{'fun',Anno}=A,{atom,_,Func}=B,{'/',_}=C,{integer,_,Arity}=D| Rest],Acc) ->
+                            case erl_internal:bif(Func, Arity) of
+                                true ->
+                                    F(Rest, [D,C,B,{':',A},{atom,Anno,'erlang'},A | Acc]);
+                                false ->
+                                    F(Rest, [D,C,B,{':',A},{atom,Anno,'shell_default'},A | Acc])
+                            end;
+                            F([H|Rest], Acc) ->
+                                F(Rest, [H | Acc]);
+                            F([], Acc) ->
+                                lists:reverse(Acc)
+                        end,
+                        Toks = LocalFunFixer(Toks0, []),
+                          %% NOTE: we can handle function definitions, records and type declarations
                           %% but this cannot be handled by the function which only expects erl_parse:abstract_expressions()
                           %% for now just pattern match against those types and pass the string to shell local func.
                           case Toks of
@@ -300,7 +323,8 @@ get_command(Prompt, Eval, Bs, RT, FT, Ds) ->
                                   case Atom of
                                       record -> SpecialCase(rd);
                                       spec -> SpecialCase(ft);
-                                      type -> SpecialCase(td)
+                                      type -> SpecialCase(td);
+                                      _ -> erl_eval:extended_parse_exprs(Toks)
                                   end;
                               [{atom, _, FunName}, {'(', _}|_] ->
                                   case erl_parse:parse_form(Toks) of
@@ -973,7 +997,7 @@ not_restricted(exit, []) ->
     true;
 not_restricted(fl, []) ->
     true;
-not_restricted(fd, [_]) ->
+not_restricted(fd, [_,_]) ->
     true;
 not_restricted(ft, [_]) ->
     true;
@@ -999,6 +1023,8 @@ not_restricted(rr, [_,_]) ->
     true;
 not_restricted(rr, [_,_,_]) ->
     true;
+not_restricted(v, [_]) ->
+    true;
 not_restricted(_, _) ->
     false.
 
